C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
      module orbital
C
C   Jellium cluster version
C
       use highlevel
       use midlevel
       implicit none

C   Calculates single electron wavefunctions and its derivatives.
C
C   The single-electron wave functions introduced in the determinant
C   are analytical fittings to a DFT/LDA numerical calculation,
C   performed using the exchange-correlation potential of Ceperley/Adler.
C
C   MPHI=number of coefficients used to fit the DFT wave functions
C   NORB=number of orbitals
C   NELORB(NE)=maps the electron index into the set of orbitals
C   Slater parameter SLAP adds an additional compression to the
C   single-particle wavefunctions, according to f(r)~*exp(-c9*r/SLAP).
C
       integer,parameter,public          :: MPHI=9,NORB=9
       integer,dimension(NE),public      :: NELORB
       real(dp),public                   :: SLAP
       real(kind=dp),public,dimension(NORB) :: PSINEW
C
C   Internal quantities
C   LMOM(NE)=maps the electron index into the angular momentum value
C               of the orbital
C   PHIAA contains the numerical coefficients need to obtain the DFT
C   wave functions
C   COEFANG contains angular coefficients for the DFT wave functions
C
       integer,private,dimension(NE) :: LMOM
       real(kind=dp),private,dimension(0:MPHI,NORB) :: phiaa
       real(kind=dp),private,dimension(NORB) :: coefang
C
       data slap / 0.96_dp /

       data lmom(1:NE)/ 0, 1, 1, 1, 2, 2, 2, 2, 2,
     +                  0, 1, 1, 1, 2, 2, 2, 2, 2 /
C
       data phiaa(0:MPHI,1)/ 0.0303556_dp, 0.0361476_dp,
     +  0.0001329978_dp, 0.0331187_dp, -0.02065_dp, 0.00927797_dp,
     +  -0.00190247_dp, 0.000205277_dp, -0.00000650376_dp,
     +   1.05859_dp /
C
       data phiaa(0:MPHI,2)/ 0.00296738_dp, 0.0208747_dp,
     +  -0.0323465_dp, 0.0416907_dp, -0.0227924_dp, 0.00729652_dp,
     +  -0.00114107_dp, 0.0000827663_dp, -0.000000155754_dp,
     +   1.23709_dp /
C
       data phiaa(0:MPHI,5)/ 0.0049356_dp, -0.0150948_dp,
     +  0.0219221_dp, -0.0143297_dp, 0.00529139_dp, -0.00107669_dp,
     +  0.00012213_dp, -0.00000663367_dp, 0.000000139151_dp,
     +   1.01174_dp /
C
       data coefang(1:NORB)/ 0.28209479177_dp, 0.4886025119_dp,
     +  0.4886025119_dp, 0.4886025119_dp, 0.31539156525_dp,
     +  1.09254843059_dp, 1.09254843059_dp, 1.09254843059_dp,
     +  0.5462742153_dp /
C
C   Specify variables
       public :: INITORB,ORBWAV,ORBDER
      contains
C-----------------------------------------------------------------------
       subroutine INITORB
C  1. Maps the electron index onto the set of orbitals through the
C     index array i=NELORB(k), k=electron index
C     For a jellium cluster of 18 electrons:
C             up-spin electron IE=1, IEES=1 occupies 1st orbital
C             up-spin electron IE=2, IEES=2 occupies 2nd orbital
C             up-spin electron IE=3, IEES=3 occupies 3rd orbital
C             up-spin electron IE=4, IEES=4 occupies 4th orbital
C             up-spin electron IE=5, IEES=5 occupies 5th orbital
C             up-spin electron IE=6, IEES=6 occupies 6th orbital
C             up-spin electron IE=7, IEES=7 occupies 7th orbital
C             up-spin electron IE=8, IEES=8 occupies 8th orbital
C             up-spin electron IE=9, IEES=9 occupies 9th orbital
C           down-spin electron IE=10,IEES=1 occupies 1st orbital
C           down-spin electron IE=11,IEES=2 occupies 2nd orbital
C           down-spin electron IE=12,IEES=3 occupies 3rd orbital
C           down-spin electron IE=13,IEES=4 occupies 4th orbital
C           down-spin electron IE=14,IEES=5 occupies 5th orbital
C           down-spin electron IE=15,IEES=1 occupies 6th orbital
C           down-spin electron IE=16,IEES=2 occupies 7th orbital
C           down-spin electron IE=17,IEES=3 occupies 8th orbital
C           down-spin electron IE=18,IEES=4 occupies 9th orbital
C
C  2. Initializes the arrays with the coefficients necessary
C     to obtain the single-particle wave functions.
C
C  3. Calculates initial wavefunction matrix PSIMAT(i,k,s) with
C     i=orbital index, k= electron index, s= spin index
C
        integer                   :: j,jj,ies,hi
        integer,dimension(NE)     :: hnelorb
        real(dp)                  :: hh,r
        real(dp),dimension(NORB)  :: psi
        data hnelorb / 1, 2, 3, 4, 5, 6, 7, 8, 9,
     +                 1, 2, 3, 4, 5, 6, 7, 8, 9/

C     Index array to associate each electron with a wavefunction

        NELORB(1:NE)=hnelorb(1:NE)

C
C     Loop to initalize the coefficients array
C
      do j=0,MPHI
        phiaa(j,3)=phiaa(j,2)
        phiaa(j,4)=phiaa(j,2)
        phiaa(j,6)=phiaa(j,5)
        phiaa(j,7)=phiaa(j,5)
        phiaa(j,8)=phiaa(j,5)
        phiaa(j,9)=phiaa(j,5)
      end do

C     The matrix psi_i(k) of the wavefunction is determined.
C     In the columns the index runs over the orbitals i for a
C     fixed electron index k.
C     But one has to discriminate between both spin directions.
C     One begins with spin-up electrons counting them
C     up to the number of electrons NES(1) with this spin.
C     Subsequently follow the spin-down electrons
C     in a new matrix up to NES(2).

         do j=1,NE
          ies=1
          if (j > NES(1)) ies=2
          hi=(ies-1)*NES(1)
          call ORBWAV(RE(1:3,j),psi)
          do jj=1,NES(ies)
           PSIMAT(jj,j-hi,ies)=psi(NELORB(hi+jj))
          end do
         end do
       end subroutine INITORB
C-----------------------------------------------------------------------
       subroutine ORBWAV(r,psi)
        real(kind=dp),intent(in),dimension(3)      :: r
        real(kind=dp),intent(out),dimension(NORB) :: psi
C
C   Obtains the DFT single-particle wave functions
C   of the jellium cluster through an analytical fitting of the
C   radial part.
C   There are nine different wave functions:
C   1.- 1s wave function
C   2.- 1px wave function
C   3.- 1py wave function
C   4.- 1pz wave function
C   5.- 1d,z2 wave function
C   6.- 1d,xy wave function
C   7.- 1d,xz wave function
C   8.- 1d,yz wave function
C   9.- 1d,x2-y2 wave function
C
        integer                              :: j
        real(kind=dp)                        :: rr

        rr=dsqrt(r(1)**2+r(2)**2+r(3)**2)
        if (rr < EMACH) rr=EMACH
C
        psi(1:NORB) = 0.0_dp
C	
	    do j=0,MPHI-1
         psi(1:NORB)=psi(1:NORB)+phiaa(j,1:NORB)*(rr**dble(j))
        end do

        psi(1:NORB)=coefang(1:NORB)*psi(1:NORB)
     +             *dexp(-phiaa(MPHI,1:NORB)*rr/SLAP)
     +             *rr**dble(lmom(1:NORB))
C
C       We have added the overall angular coefficient, but we still have
C       to add the spacial part of the angular dependence if necessary
C       (p- and d- symmetries):
C
        psi(2)=psi(2)*r(1)/rr
        psi(3)=psi(3)*r(2)/rr
        psi(4)=psi(4)*r(3)/rr

        psi(5)=psi(5)*(2.0_dp*r(3)*r(3)-r(1)*r(1)-r(2)*r(2))/rr/rr
        psi(6)=psi(6)*r(1)*r(2)/rr/rr
        psi(7)=psi(7)*r(1)*r(3)/rr/rr
        psi(8)=psi(8)*r(2)*r(3)/rr/rr
        psi(9)=psi(9)*(r(1)*r(1)-r(2)*r(2))/rr/rr
C
       end subroutine ORBWAV
C
C-----------------------------------------------------------------------
C
       subroutine ORBDER(r,psi,pgr,pla)
C
C   Gradient and Laplacian of the single-particle wave functions.
C
        real(dp),intent(in),dimension(3) :: r
        real(dp),intent(out),dimension(NORB)    :: psi,pla
        real(dp),intent(out),dimension(3,NORB)  :: pgr
C
C    QUESTION= But we do not need 'Psi' here!! Could we just remove it?
C
        integer                        :: j,k
C
C       The arrays ww, ww1, and ww2, temporarily save the
C       radial wave function, its first radial derivative,
C       and its second radial derivative respectively
C
        real(dp),dimension(NORB)       :: ww,ww1,ww2
        real(kind=dp)                  :: rr,wtmp,wtmp2,ftmp
        real(dp),dimension(3)          :: gftmp

        rr=dsqrt(r(1)**2+r(2)**2+r(3)**2)
        if (rr < EMACH) rr=EMACH

C
C     We calculate the radial wave functions and their first and
C     second radial derivatives
C

        ww(1:NORB)=0.0_dp
        ww1(1:NORB)=0.0_dp
        ww2(1:NORB)=0.0_dp
        gftmp(1:3)=0.0_dp

        do k=1,NORB
          do j=0,MPHI-1
           ww(k)=ww(k)+phiaa(j,k)*(rr**dble(j))
           if (j.ge.(1-lmom(k))) ww1(k)=ww1(k)+dble(j+lmom(k))*
     +                       phiaa(j,k)*(rr**dble(j+lmom(k)-1))
           if (j.ge.(2-lmom(k))) ww2(k)=ww2(k)+dble(j+lmom(k))*
     +                          dble(j+lmom(k)-1)*phiaa(j,k)*
     +                          (rr**dble(j+lmom(k)-2))
          end do
          ww(k)=ww(k)*dexp(-phiaa(MPHI,k)*rr/SLAP)*rr**dble(lmom(k))
          wtmp=ww1(k)
          ww1(k)=(ww1(k)*dexp(-phiaa(MPHI,k)*rr/SLAP))
     +        -phiaa(MPHI,k)*ww(k)/SLAP
          ww2(k)=(ww2(k)-2.0_dp*phiaa(MPHI,k)*wtmp/SLAP)
     +         *dexp(-phiaa(MPHI,k)*rr/SLAP)
     +         +phiaa(MPHI,k)*phiaa(MPHI,k)*ww(k)/SLAP/SLAP
        end do

C
C      Gradient and Laplacian of the 1s orbital
C

        psi(1)=coefang(1)*ww(1)
        do j=1,3
         pgr(j,1)=coefang(1)*ww1(1)*r(j)/rr
        end do
        pla(1)=coefang(1)*(ww2(1)+2.0_dp*ww1(1)/rr)

C
C      Gradient and Laplacian of the 1p orbitals
C

        wtmp=ww2(2)/rr+2.0_dp*ww1(2)/rr/rr-2.0_dp*ww(2)/rr/rr/rr
        do k=2,4
         psi(k)=coefang(k)*ww(k)*r(k-1)/rr
         do j=1,3
          pgr(j,k)=(ww1(k)/rr/rr-ww(k)/rr/rr/rr)*r(j)*r(k-1)
          if (j.eq.(k-1)) pgr(j,k)=pgr(j,k)+ww(k)/rr
          pgr(j,k)=coefang(k)*pgr(j,k)
         end do
         pla(k)=coefang(k)*r(k-1)*wtmp
        end do
C
C      Gradient and Laplacian of the 1d orbitals
C
        wtmp=ww1(5)/rr/rr/rr-2.0_dp*ww(5)/rr/rr/rr/rr
        wtmp2=ww2(5)/rr/rr+2.0_dp*ww1(5)/rr/rr/rr
     +       -6.0_dp*ww(5)/rr/rr/rr/rr
        do k=5,9
         if (k.eq.5) then
          ftmp=2.0_dp*r(3)*r(3)-r(1)*r(1)-r(2)*r(2)
          gftmp(1)=-2.0_dp*r(1)
          gftmp(2)=-2.0_dp*r(2)
          gftmp(3)=4.0_dp*r(3)
         else if (k.eq.6) then
          ftmp=r(1)*r(2)
          gftmp(1)=r(2)
          gftmp(2)=r(1)
          gftmp(3)=0.0_dp
         else if (k.eq.7) then
          ftmp=r(1)*r(3)
          gftmp(1)=r(3)
          gftmp(2)=0.0_dp
          gftmp(3)=r(1)
         else if (k.eq.8) then
          ftmp=r(2)*r(3)
          gftmp(1)=0.0_dp
          gftmp(2)=r(3)
          gftmp(3)=r(2)
         else if (k.eq.9) then
          ftmp=r(1)*r(1)-r(2)*r(2)
          gftmp(1)=2.0_dp*r(1)
          gftmp(2)=-2.0_dp*r(2)
          gftmp(3)=0.0_dp
         end if
         psi(k)=coefang(k)*ww(k)*ftmp/rr/rr
         do j=1,3
          pgr(j,k)=wtmp*ftmp*r(j)+ww(k)*gftmp(j)/rr/rr
          pgr(j,k)=coefang(k)*pgr(j,k)
         end do
         pla(k)=coefang(k)*ftmp*wtmp2
        end do

       end subroutine ORBDER
C
      end module orbital
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
