C-----------------------------------------------------------------------
      module jastrow
       use highlevel
       use midlevel
C  Calculates the change of the Jastrow two-particle potential and
C  updates the derivatives. The Jastrow factor J is written as
C  J=exp(-\sum_k u) with u as in the subroutines.
C  GRJAS(3,NE,IES)=  Gradient of J divided by J for spin IES
C  LAPJAS(NE,IES)= Laplace of J divided by J for spin IES
C  QJ(IES)= J^new/J^old, Jastrow part of acceptance for spin IES,
C  to be squared to yield the rates
C  calculated for actual electron, IE.
C  CJAS= only Jastrow constant for exponential like Jastrow factor
C  BETA1,BETA2= Additional constants for atomic like Jastrow factor,
C  which introduce a short range repulsion (attraction) for
C  positive (negative) values. Set BETA1=0.0 to cancel the latter part.
       implicit none
       public :: INITJAS,JASEXPATOM,JASEXPSOLID,JASEXPSOLID1
       real(kind=dp),public:: CJAS,CJASOPT,BETA1,BETA2,CJAS1
       real(kind=dp),dimension(2),public      ::QJ,QJOPT
       real(kind=dp),dimension(NE,2),public   :: LAPJAS,LAPJASOLD
       real(kind=dp),dimension(3,NE,2),public :: GRJAS,GRJASOLD
      contains
C-----------------------------------------------------------------------
      subroutine INITJAS
       BETA1=0.0_dp
       BETA2=0.0_dp
      end subroutine INITJAS
C-----------------------------------------------------------------------
      subroutine JASEXPATOM
C  Determines Jastrow exponent and derivatives for a form
C  u=F/2/(1+r/F)*(delta_{s,-s'}+1/2*delta_{s,s'})+
C    BETA_1exp(-BETA2*r^2). From the derivatives
C  the kinetic energy contribution of the Jastrow factor
C  is obtained.
        integer :: i,k,n
        real(dp) :: as,jasu,jasdif,u2d,u2do,woo,won
        real(dp),dimension(3) :: u1d,u1do
        jasu=0.0_dp
        jasdif=0.0_dp
        u1d(1:3)=0.0_dp
        u1do(1:3)=0.0_dp
        u2d=0.0_dp
        u2do=0.0_dp
        call RDIST(RE,RENEW)
        ielekat:do k=1,NE
         if (k .eq. IE) then
          cycle ielekat
         end if
         as=0.5_dp     ! for equal spins
         if (((IES == 1) .and. (k > NES(1))) .or.
     &       ((IES == 2) .and. (k <= NES(1)))) as =1.0_dp
         woo = DIST(4,IE,k)
         won = DISTNEW(4,IE,k)
         jasdif=jasdif+(as/2.0_dp/(1.0_dp+won/CJAS)-
     &       as/2.0_dp/(1.0_dp+woo/CJAS))*CJAS+
     &       BETA1*(dexp(-BETA2*won**2)-dexp(-BETA2*woo**2))
         jasu=jasu+as/2.0_dp/(1.0_dp+won/CJAS)*CJAS+
     &       BETA1*dexp(-BETA2*won**2)
         u1d(1:3)=u1d(1:3)-as*DISTNEW(1:3,IE,k)/won/
     &            (1.0_dp+won/CJAS)**2/2.0_dp-
     &       2.0_dp*DISTNEW(1:3,IE,k)*BETA1*BETA2*dexp(-BETA2*won**2)
         u1do(1:3)=u1do(1:3)-as*DIST(1:3,IE,k)/woo/
     &            (1.0_dp+woo/CJAS)**2/2.0_dp-
     &       2.0_dp*DIST(1:3,IE,k)*BETA1*BETA2*dexp(-BETA2*woo**2)
         u2d=u2d-as/won/(1.0_dp+won/CJAS)**3+
     &    BETA1*BETA2*dexp(-BETA2*won**2)*(4.0_dp*BETA2*won**2-6.0_dp)
         u2do=u2do-as/woo/(1.0_dp+woo/CJAS)**3+
     &    BETA1*BETA2*dexp(-BETA2*woo**2)*(4.0_dp*BETA2*woo**2-6.0_dp)
        end do ielekat
        QJ(IES) = dexp(-jasdif)
        GRJAS(1:3,IEES,IES)=-u1d(1:3)
        GRJASOLD(1:3,IEES,IES)=-u1do(1:3)
        LAPJAS(IEES,IES)=-u2d+(u1d(1)**2+u1d(2)**2+u1d(3)**2)
        LAPJASOLD(IEES,IES)=-u2do+(u1do(1)**2+u1do(2)**2+u1do(3)**2)
       end subroutine JASEXPATOM
C-----------------------------------------------------------------------
       subroutine JASEXPSOLID
C  Determines Jastrow exponent and derivatives for a form
C  u=as*CJAS**2/r*(1-e^(-r/CJAS)), as=0.5 for parallel spin and
C  as=1.0 for antiparallel spins. From the derivatives
C  the kinetic energy contribution of the Jastrow factor
C  is obtained. Here, the harmonic potential is not included to
C  get a continuous derivative (Gradient). The exponential term
C  of the Jastrow factor is neglected in the Ewald sum, because
C  of rapid decay for distances beyond the Wigner-Seitz cell.
        integer :: i,k,n
        real(dp) :: as,jasu,jasdif,u2d,u2do,woo,won
        real(dp),dimension(3) :: u1d,u1do
        jasu=0.0_dp
        jasdif=0.0_dp
        u1d(1:3)=0.0_dp
        u2d=0.0_dp
        u1do(1:3)=0.0_dp
        u2do=0.0_dp
        call RDIST(RE,RENEW)
        ieleksol:do k=1,NE
         if (k .eq. IE) then
          cycle ieleksol
         end if
         as=0.5_dp     ! for equal spins
         if (((IES == 1) .and. (k > NES(1))) .or.
     &       ((IES == 2) .and. (k <= NES(1)))) as =1.0_dp
         woo = DIST(4,IE,k)
         won = DISTNEW(4,IE,k)
         jasdif=jasdif+as*(((1.0_dp-dexp(-won/CJAS))/won)-
     &         ((1.0_dp-dexp(-woo/CJAS))/woo))*CJAS**2
         jasu=jasu+
     &        as*((1.0_dp-dexp(-won/CJAS))/won)*CJAS**2
         u1d(1:3)=u1d(1:3)-as*DISTNEW(1:3,IE,k)/won**3*(1.0_dp-
     &       dexp(-won/CJAS)*(1.0_dp+won/CJAS))*CJAS**2
         u1do(1:3)=u1do(1:3)-as*DIST(1:3,IE,k)/woo**3*(1.0_dp-
     &       dexp(-woo/CJAS)*(1.0_dp+woo/CJAS))*CJAS**2
         u2d=u2d-as/won*dexp(-won/CJAS)
         u2do=u2do-as/woo*dexp(-woo/CJAS)
        end do ieleksol
        QJ(IES) = dexp(-jasdif)
        GRJAS(1:3,IEES,IES)=-u1d(1:3)
        GRJASOLD(1:3,IEES,IES)=-u1do(1:3)
        LAPJAS(IEES,IES)=-u2d+sum (u1d(1:3)**2)
        LAPJASOLD(IEES,IES)=-u2do+sum (u1do(1:3)**2)
      end subroutine JASEXPSOLID
C-----------------------------------------------------------------------
       subroutine JASEXPSOLID1
C  Determines Jastrow exponent and derivatives for a form
C  u=as*CJAS**2/r*(1-e^(-r/CJAS)), as=0.5 for parallel spin and
C  as=1.0 for antiparallel spins. From the derivatives
C  the kinetic energy contribution of the Jastrow factor
C  is obtained. Here, the harmonic potential is not included to
C  get a continuous derivative (Gradient). The exponential term
C  of the Jastrow factor is neglected in the Ewald sum, because
C  of rapid decay for distances beyond the Wigner-Seitz cell.
        integer :: i,k,n
        real(dp) :: as,jasu,jasdif,u2d,u2do,woo,won
        real(dp),dimension(3) :: u1d,u1do
        jasu=0.0_dp
        jasdif=0.0_dp
        u1d(1:3)=0.0_dp
        u2d=0.0_dp
        u1do(1:3)=0.0_dp
        u2do=0.0_dp
        call RDIST(RE,RENEW)
        CJAS1 = CJASOPT
        ieleksol:do k=1,NE
         if (k .eq. IE) then
          cycle ieleksol
         end if
         as=0.5_dp     ! for equal spins
         if (((IES == 1) .and. (k > NES(1))) .or.
     &       ((IES == 2) .and. (k <= NES(1)))) as =1.0_dp
         woo = DIST(4,IE,k)
         won = DISTNEW(4,IE,k)
         jasdif=jasdif+as*(((1.0_dp-dexp(-won/CJAS1))/won)-
     &         ((1.0_dp-dexp(-woo/CJAS1))/woo))*CJAS1**2
         jasu=jasu+
     &        as*((1.0_dp-dexp(-won/CJAS1))/won)*CJAS1**2
         u1d(1:3)=u1d(1:3)-as*DISTNEW(1:3,IE,k)/won**3*(1.0_dp-
     &       dexp(-won/CJAS1)*(1.0_dp+won/CJAS1))*CJAS1**2
         u1do(1:3)=u1do(1:3)-as*DIST(1:3,IE,k)/woo**3*(1.0_dp-
     &       dexp(-woo/CJAS1)*(1.0_dp+woo/CJAS1))*CJAS1**2
         u2d=u2d-as/won*dexp(-won/CJAS1)
         u2do=u2do-as/woo*dexp(-woo/CJAS1)
        end do ieleksol
        QJ(IES) = dexp(-jasdif)
        GRJAS(1:3,IEES,IES)=-u1d(1:3)
        GRJASOLD(1:3,IEES,IES)=-u1do(1:3)
        LAPJAS(IEES,IES)=-u2d+sum (u1d(1:3)**2)
        LAPJASOLD(IEES,IES)=-u2do+sum (u1do(1:3)**2)
      end subroutine JASEXPSOLID1
C-----------------------------------------------------------------------
      end module jastrow
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
