C-----------------------------------------------------------------------
      module ewald
C  Routines for the Ewald summation
C
C  The potentials are stored on a symmetry part of the cube representing
C  the unit cell of the sc superlattice. Only one sixth of the cube
C  in which the coordinates of the points are positive and have values
C  ordered as R(1)>=R(2)>=R(3) is taken. Respective transformations have
C  always to be applied before associating values of stored functions
C  with a position. The respective points are taken from the positive
C  octant of the cube with center at (0,0,0) by using the absolute
C  values (i.e. a rotation) of coordinates and by an exchange of
C  coordinates (also belonging to the cubic group) to get the above
C  ordering.
       use highlevel
       use midlevel
       implicit none
C  MLAT= Number of lattices, here only cubic ones
C  MAXIR= Maximum index in one dimension of the mesh in the wedge of
C  direct space for tabulation
C  MAXSR= (2231)total number of points in the wedge including adjacent
C  points from the interior of the cube being not in the wedge
C  MAX2SR= (9261) total number of points in the octant incl. adjacent
C  MINR,MAXR= defines the cube of tabulation
C  DR= coordinate distance between two tabulated points
C  VEWTAB= Ewald lattice potential as tabulated in VEWDAT.DAT
C  VEEEWT= Interpolated potential from tabulated Ewald potential
       integer,parameter,public :: MAXIR=21
       integer,parameter,public :: MAXSR=MAXIR*(MAXIR+1)*(MAXIR+8)/6-2
       integer,parameter,public :: MAX2SR=MAXIR*MAXIR*MAXIR
       real(dp),dimension(MAXSR,MLAT),public :: VEWTAB
       real(dp),dimension(MAX2SR,MLAT),public :: VEW2TAB
       real(dp),dimension(NE,NE),public :: VEEEWT
       real(dp),dimension(3,NE,NE),public :: VEEEW1T,VEES1T
       logical,public  :: BVEEEW,BVEES1,BVEEEW1,BRKE6,BRKEP,BREE6,BREEP
       real(dp),parameter,private :: MINR=0.0,MAXR=0.5,
     &       DR=((MAXR-MINR)/(MAXIR-1))
       public  ::  INITVEWALD,VEW,WEDGE,OCTANT,VEWGR,TAB_EWALD
      contains
C-----------------------------------------------------------------------
      subroutine INITVEWALD
C  Initializes the Ewald lattice potential VEWTAB by read-in from
C  file VEWDAT.DAT.
       integer  ::  in,ig
       open(45,file='VEWDAT.DAT',status='OLD')
       open(46,file='VEW2DAT.DAT',status='OLD')
       do ig=1,MLAT
        do in=1,MAXSR
         read (45,*,END=999) VEWTAB(in,ig)
        end do
        do in=1,MAX2SR
         read (46,*,END=999) VEW2TAB(in,ig)
        end do
       end do
       write (39,*) 'Initial VEW OK !'
999    continue
       close(45)
       close(46)
C  Initializes VEEEWT
       VEEEWT = 0.0_dp
       VEES1T = 0.0_dp
       VEEEW1T = 0.0_dp
       BVEEEW = .false.
       BVEES1 = .false.
       BVEEEW1 = .false.
C  Initialize the index arrays
        BRKE6 = .FALSE.
        BRKEP = .FALSE.
        BREE6 = .FALSE.
        BREEP = .FALSE.
      end subroutine INITVEWALD
C-----------------------------------------------------------------------
      subroutine WEDGE(rj,ir,wr)
C  Calculates the corners of a cubic mesh which enclose a specific
C  image of the point rj put to the reduced wedge of the superlattice
C  sc cell. The image is obtained from a mapping by cubic
C  transformations, see module header for a description.
       real(dp),intent(in),dimension(3) :: rj
       integer,intent(out),dimension(8) :: ir
       real(dp),intent(out),dimension(8) :: wr
C  rj= position in sc superlattice cube [-DR(MAXIR-1),+DR(MAXIR-1)]^3
C  ir= mesh index for the 8 corners of the enclosing cube in the 1/6th
C  wedge. Mesh points adjacent but beyond the surface are included, but
C  not beyond index MAXIR, to include the full cubes necessary to
C  interpolate also the surface points.
C  wr= 8 weights for interpolating the true position from the corners.
C  The weights and the index of the 8 mesh points are output.
        integer :: i,i1,i2,i3
        integer,dimension(3,0:1) :: iq
        real(dp) :: h1,h2
        real(dp),dimension(3) :: rq
        real(dp),dimension(3,0:1) :: wq
C  Put rj into the positive octant
        rq = abs(rj)
C  Exchange coordinates for sorting into the wedge rq(1)>=rq(2)>=rq(3)
        h1 = minval (rq)
        h2 = maxval (rq)
        rq(2) = minval (rq,rq-h1 >= 0)
        rq(1) = h2
        rq(3) = h1
C  Choose the mesh points iq enclosing point rq and determine the
C  distance wq between them and rq in units of DR. The indices count
C  from 1 upwards, e.g. rq=(0,0,0) being associated with iq=(1,1,1).
C  Define the i=1,8 corner points of the rq enclosing cube by the
C  coordinates.
C       (iq(1,0),iq(2,0),iq(3,0)) for i=1
C       (iq(1,0),iq(2,0),iq(3,1)) for i=2
C       (iq(1,0),iq(2,1),iq(3,0)) for i=3
C       (iq(1,0),iq(2,1),iq(3,1)) for i=4
C       (iq(1,1),iq(2,0),iq(3,0)) for i=5
C       (iq(1,1),iq(2,0),iq(3,1)) for i=6
C       (iq(1,1),iq(2,1),iq(3,0)) for i=7
C       (iq(1,1),iq(2,1),iq(3,1)) for i=8
C
        iq(1:3,0) = int(rq(1:3)/DR) + 1
        iq(1:3,1) = iq(1:3,0) + 1
        wq(1:3,0) = (DR*dble(iq(1:3,0))-rq(1:3))/DR
        wq(1:3,1) = (rq(1:3)-DR*dble(iq(1:3,0)-1))/DR
C  Put the mesh points above MAXIR onto the border with
C  weight zero keeping only that with weight one which was on
C  the surface.
        do i=1,3
         if (iq(i,1) > MAXIR) then
          iq(i,0:1) = MAXIR
          wq(i,0) = 1.0_dp
          wq(i,1) = 0.0_dp
         end if
        end do
C  Associate its mesh index with each of the 8 corner points
C  of the enclosing cube. This can be seen also as a definition of
C  the whole mesh when the 3-dimensional indes runs over all values
C  of the reduced wedge of the total cube. Then instead of the local
C  index (i1,i2,i3) the global index (iq(1),iq(2),iq(3)) runs
C  according to:
C  i=0
C  do iq(1)=1,MAXIR
C     iq(2)=1,(iq(1)+1)    without (MAXIR+1)
C     iq(3)=1,(iq(2)+1)   without (iq(1)+2)
C     i=i+1
C     ir(i)=
C     iq(3)+((iq(2)+2)*(iq(2)-1))/2+((iq(1)-1)*(iq(1)*(iq(1)+7)+6)/6
C  end do
C
C  The weights are chosen from linear interpolation in each coordinate
C  direction, i.e. as the product of the coordinates' deviation
C  (in units of DR)of the point under consideration from a corner point
C  Any function f we have : f(rj(1:3))=\sum^8_i=1 wr(i)*f(r_{ir(i)}) as
C  a sum over the corners.
        i = 0
        do i1=0,1
         do i2=0,1
          do i3=0,1
           i = i+1
           ir(i) = iq(3,i3)
     &        + ((iq(2,i2)+2)*(iq(2,i2)-1))/2
     &        + ((iq(1,i1)-1)*(iq(1,i1)*(iq(1,i1)+7)+6))/6
           wr(i) = wq(1,i1)*wq(2,i2)*wq(3,i3)
          end do
         end do
        end do
      end subroutine WEDGE
C-----------------------------------------------------------------------
      subroutine OCTANT(rj,ir,wr)
C  Simpler than WEDGE which would be only 1/6th of the octant
C  which is used here. But this program is for the derivative of the
C  Ewald potential which needs a coordinate exchange to obtain all
C  three gradient components from the one which is tabulated. In
C  transforming back respective minus signs apply also because of
C  treating only one octant.
C  Calculates the corners of a cubic mesh which enclose a specific
C  point rj of the superlattice sc cell. It uses from cubic symmetry
C  the mirrors of the coordinate axes to calculate only one octant
C  of the cube. The origin (0,0,0)
C  is shifted to index ir(1)=1
       real(dp),intent(in),dimension(3) :: rj
       integer,intent(out),dimension(8) :: ir
       real(dp),intent(out),dimension(8) :: wr
C
       integer :: i,i1,i2,i3
       integer,dimension(3,0:1) :: iq
       real(dp),dimension(3) :: rq
       real(dp),dimension(3,0:1) :: wq
C  Rotate the point into the positive octant of the sc superlattice
C  unit cell and determine the small cube of the interpolation mesh.
       rq(1:3) = dabs(rj(1:3))
       iq(1:3,0) = int(rq(1:3)/DR) + 1
       iq(1:3,1) = iq(1:3,0) + 1
       wq(1:3,0) = (DR*dble(iq(1:3,0))-rq(1:3))/DR
       wq(1:3,1) = (rq(1:3)-DR*dble(iq(1:3,0)-1))/DR
       do i=1,3
        if (iq(i,1) > MAXIR) then
         iq(i,0:1) = MAXIR
         wq(i,0) = 1.0_dp
         wq(i,1) = 0.0_dp
        end if
       end do
C  Here the index ir(i) is counted by i1 as the smallest unit contrary
C  to the WEDGE routine where i3 counts the smallest steps.
       i = 0
       do i1=0,1
        do i2=0,1
         do i3=0,1
          i = i+1
          ir(i) = ((iq(3,i3)-1)*MAXIR+(iq(2,i2)-1))*MAXIR + iq(1,i1)
          wr(i) = wq(1,i1)*wq(2,i2)*wq(3,i3)
         end do
        end do
       end do
      end subroutine OCTANT
C-----------------------------------------------------------------------
      function VEW(latty,rj)
C  Calculates the Ewald sum, \sum_R [1/|rj-R|], over all lattice points
C  of lattice type latty
C  and subtracts the jellium background contribution. Works for
C  sc,fcc,bcc lattices (LATTY=1,2,3). However, in the first step
C  \sum_R (1/|rj-R|)-jellium -1/|rj| is determined by interpolation with
C  subroutine WEDGE, such that 1/|rj| must be added later to obtain VEW.
       real(dp) :: VEW
       integer,intent(in) :: latty
       real(dp),intent(in),dimension(3) :: rj
C
       integer,dimension(8) :: ir
       real(dp),dimension(8) :: wr
       call WEDGE(rj,ir,wr)
       VEW = sum (wr(1:8)*VEWTAB(ir(1:8),latty))
      end function VEW
C-----------------------------------------------------------------------
      subroutine VEWGR(latty,rj,grad)
C  Calculates the gradient of VEW, i.e. \sum_R [(rj-R)/|rj-R|^3],
C  over all lattice points of lattice type latty. In contrast to VEW,
C  the routine uses the full cube without reducing to the wedge as
C  it is done in VEW. Note that only one component has been tabulated.
C  The two remaining components have to be obtained by the cubic group,
C  i.e. in simple words by an axes exchange.
       integer,intent(in) :: latty
       real(dp),intent(in),dimension(3) :: rj
       real(dp),intent(out),dimension(3) :: grad
C
       integer :: ii
       integer,dimension(8) :: ir,hi0
       integer,dimension(3,8) :: hi,hir
       real(dp),dimension(8) :: wr
C
       call OCTANT(rj,ir,wr)
C  Coordinates are exchanged to obtain the whole gradient.
C  Note the integer arithmetic.
C  The 1st component has been already calculated from the table
       hir(1,1:8) = ir(1:8)
C  First the 2nd component is generated
       hi0(1:8) = ir(1:8) -1
       hi(1,1:8) = mod (hi0(1:8),MAXIR)
       hi0(1:8) = hi0(1:8)/MAXIR
       hi(2,1:8) = mod (hi0(1:8),MAXIR)
       hi0(1:8) = hi0(1:8)/MAXIR
       hi(3,1:8) = mod (hi0(1:8),MAXIR)
       hi0(1:8) = hi(1,1:8)
       hi(1,1:8) = hi(2,1:8)
       hi(2,1:8) = hi0(1:8)
       hir(2,1:8) = (hi(3,1:8)*MAXIR+hi(2,1:8))*MAXIR + hi(1,1:8) + 1
C  At last, the 3rd component is generated
       hi0(1:8) = ir(1:8) -1
       hi(1,1:8) = mod (hi0(1:8),MAXIR)
       hi0(1:8) = hi0(1:8)/MAXIR
       hi(2,1:8) = mod (hi0(1:8),MAXIR)
       hi0(1:8) = hi0(1:8)/MAXIR
       hi(3,1:8) = mod (hi0(1:8),MAXIR)
       hi0(1:8) = hi(1,1:8)
       hi(1,1:8) = hi(3,1:8)
       hi(3,1:8) = hi0(1:8)
       hir(3,1:8) = (hi(3,1:8)*MAXIR+hi(2,1:8))*MAXIR + hi(1,1:8) + 1
C  Now we are ready for the whole gradient
       do ii=1,3
        grad(ii) = sum (wr(1:8)*VEW2TAB(hir(ii,1:8),latty))
       end do
C       if (rj(1:3) < 0.0_dp) grad(1:3)=-grad(1:3)
C       grad = merge (-grad,+grad,mask = grad<0)
       grad = merge (-grad,+grad,mask = rj<0)
      end subroutine VEWGR
C-----------------------------------------------------------------------
      subroutine TAB_EWALD
C  Calculate tables for the Ewald sums
C  rl,gl =direct, reciprocal lattice
       real(dp),dimension(3,(2*MAXIR+1)**3) :: rl,gl
C       call MAKE_LATTICE(SULATTY,MAXIR,rl)
C       call MAKE_LATTICE(SULATTY,MAXIR,gl)
      end subroutine TAB_EWALD
C-----------------------------------------------------------------------
      end module ewald
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
