C-----------------------------------------------------------------------
      module determinant
       use highlevel
       use midlevel
       use orbital_hf
C   Lithium version
C   4.10.2009: Uses new module strategy
C   Here: update of determinant and its derivatives
C   QD(2)= determinantal acceptance ratio for both spins
C   DET(2)= actual determinant
C   ANEW= matrix inverse referring to the new determinant for each spin
C   AOLD= matrix inverse referring to the old determinant for each spin
       implicit none
       real(dp),public,dimension(2)       :: QD,DET
       real(dp),public,dimension(NE,2)    :: LAPLDET,LAPLDETOLD
       real(dp),public,dimension(3,NE,2)  :: GRADDET,GRADDETOLD
       real(dp),dimension(NE,NE,2),public :: ANEW,AOLD
       public :: SLASM,SLAKIN,DETARR

      contains
C-----------------------------------------------------------------------
      subroutine INITDET
       integer  :: j,hs
       ANEW=0.0_dp
C       write (*,'(a,(i3,e12.3,/))')'PSIMAT=',(j,PSIMAT(163,j,1),j=1,192)
       do j=1,2
        hs=NES(j)
        call DETARR(hs,PSIMAT(1:hs,1:hs,j),DET(j),ANEW(1:hs,1:hs,j))
       end do
       AOLD=ANEW
      end subroutine INITDET
C-----------------------------------------------------------------------
      subroutine SLASM(q,aold,psinew,anew)
C   Calculates according to the Sherman-Morrison formula the
C   change of the inverse of a matrix, if the global electron
C   variable IE has been moved.
C   q=  ratio new-determinant/old-determinant
C   aold= matrix inverse of the old-determinant matrix before move
C   psinew(i,k)=  new wavefunction matrix after move
C   first index refers to orbital, second index to electron number
C   anew=  matrix inverse of the new-determinant matrix after move
       real(dp),intent(in)                          :: q
       real(dp),intent(in),dimension(NES(IES),NES(IES))   :: aold
       real(dp),intent(in),dimension(NES(IES),NES(IES))  :: psinew
       real(dp),intent(out),dimension(NES(IES),NES(IES))  :: anew
C
       integer                                      :: j,ns
       real(dp),dimension(NES(IES))                    :: vsum
C   The IEES'th electron of spin IES is actually considered
       ns = NES(IES)
       if (q == 0.0_dp) then
        anew = 0.0_dp
       else
        do j=1,ns
         vsum(j) = dot_product (aold(j,1:ns),psinew(1:ns,IEES))
        end do
        vsum =-vsum/q
        do j=1,ns
         anew(1:ns,j) = aold(1:ns,j) +
     &                 vsum(1:ns)*aold(IEES,j)
        end do
        anew(IEES,1:ns) = aold(IEES,1:ns)/q
       end if
      end subroutine SLASM
C-----------------------------------------------------------------------
      subroutine SLAKIN(aold,pgrnew,planew,graddet,lapldet)
C  Calculates the contributions of the Slater determinant to the local
C  kinetic energy, i.e. gradient and laplacian, of electron IE
C  with spin  IES in one-eletron wavefunction psi(orbital). ORBDER
C  has to be called before, in order to obtain the input, pgrnew and
C  planew, at the new positions of electron IE
C  aold= inverse of determinant matrix
C  pgrnew= gradient of one-electron wavefunction at new position
C  planew= laplacian of one-electron wavefunction at new position
       real(dp),intent(in),dimension(NES(IES),NES(IES)) :: aold
       real(dp),intent(in),dimension(3,NK,NORB)         :: pgrnew
       real(dp),intent(in),dimension(NK,NORB)           :: planew
       real(dp),intent(out),dimension(3)                :: graddet
       real(dp),intent(out)                             :: lapldet
C
       integer                      :: i,js,ns,ns0,hjs
       real(dp)                     :: hpla,hhpla
       real(dp),dimension(3)        :: hpgr,hhpgr
       real(dp),dimension(3,NK,NORB)  :: pgrhynew
       real(dp),dimension(NK,NORB)    :: plahynew
       logical                      :: toolarge
       ns = NES(IES)
       ns0 = (IES-1)*NES(1)
       hpgr = 0.0_dp
       hpla = 0.0_dp
       do i= 1,3  !for hybrid states
        call LIDIMER(pgrnew(i,1:NK,1:NORB),pgrhynew(i,1:NK,1:NORB))
       end do
       call LIDIMER(planew(1:NK,1:NORB),plahynew(1:NK,1:NORB))
       do js=1,ns
        hjs = js
C  The  NK/2 2s electrons refer to the first NK/2 nuclei for spin-up
C  and the last NK/2 nuclei for spin-down
        if (js > NK) then
         hjs=js-NK+NK/2*(IES-1)
         hhpgr(1:3) = pgrhynew(1:3,hjs,NELORB(ns0+js))
         hhpla = plahynew(hjs,NELORB(ns0+js))
        else
         hhpgr(1:3) = pgrnew(1:3,hjs,NELORB(ns0+js))
         hhpla = planew(hjs,NELORB(ns0+js))
        end if
        hpgr(1:3) = hpgr(1:3) +
     &              aold(IEES,js)*hhpgr(1:3)
        hpla = hpla + aold(IEES,js)*hhpla
       end do
       graddet(1:3) = hpgr(1:3)
       lapldet = hpla
       toolarge = .false.
       if ((sum(dabs(hpgr)) > 100.0_dp) .OR.
     &         (hpla > 100.0_dp)) toolarge = .true.
C       if (toolarge) write(39,*) 'kinetic energy large grad,lapl=',
C    &              abs(sum(hpgr),hpla
      end subroutine SLAKIN
C-----------------------------------------------------------------------
      subroutine DETARR(nies,psiarr,det,anew)
C  Calculates the determinant
C  This routine is mainly for test purposes, as the determinant is
C  generated at start by random filling and always updated with help of
C  the inverse. Here, we start with unit matrix c(i,k) and determinant,
C  and update similarly by introducing at each step a column of the
C  matrix psiarr(i,k) via the inverse aold (i,k) of updated c(i,k).
       integer,intent(in)  :: nies
       real(dp),intent(in),dimension(nies,nies)  :: psiarr
       real(dp),intent(out),dimension(nies,nies) :: anew
       real(dp),intent(out)                      :: det
C
       integer   :: m,n,j,k,hnes
       real(dp)  :: q,dold,dnew
       real(dp),dimension(nies)          :: ha
       real(dp),dimension(nies,nies)     :: c,aold
       hnes = nies
       c = 0.0_dp
       do j=1,hnes
        c(j,j)=1.0_dp
       end do
       dold=1.0_dp
       aold=c
       q=0.0_dp
       do k=1,hnes
C        if(k==162) write(*,'(a,(i3,f7.3,f7.3,/))')'j,ao,psi=',
C     &      (j, aold(k,j),psiarr(j,k),j=1,hnes)
        q = dot_product (aold(k,1:hnes),psiarr(1:hnes,k))
        if (dabs(q) <= EMACH*1.e-12_dp) then
        write (*,*)'k,q=',k,q
         q=EMACH
         write(*,*)'init.determ.q=0!Linear depend. of states?'
        end if
        do m=1,hnes
         ha(m)=dot_product (aold(m,1:hnes),psiarr(1:hnes,k))
         if (m == k) then
          anew(m,1:hnes)=aold(m,1:hnes)/q
          else
          anew(m,1:hnes)=aold(m,1:hnes)-ha(m)*aold(k,1:hnes)/q
         end if
        end do
C        write(*,*)'anew=',anew(1:3,1:3)
        dnew=q*dold
        aold=anew
        dold=dnew
       end do
       det = dnew
      end subroutine DETARR
C-----------------------------------------------------------------------
      end module determinant
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
