C-----------------------------------------------------------------------
      module ewald_cal_tab
C
       use highlevel
       use midlevel
       use lattice_Lisolid
C
C  Routines for the Ewald potential
C  Two possibilities are offered: fast by table <--> slow by summing
C
C  In the latter case the Ewald potential is calculated
C  at every MC step, in the former case the potentials are stored on
C  a symmetry part of the cube representing the unit cell of
C  the sc superlattice and the table is called at every step.
C  Storage:
C  Different sets are used to store the potential and the gradient.
C  For the potential only one sixth of the cube centered at (0,0,0)
C  in which the coordinates of the points are positive and have values
C  ordered as R(1)>=R(2)>=R(3) is taken. Respective transformations have
C  always to be applied before associating a stored value
C  with a position. For the gradient the respective points are taken
C  from the positive octant of the cube with center
C  at (0,0,0). The positive octant is obtained through rotations of
C  coordinates and the above ordering by an exchange of
C  coordinates (also belonging to the cubic group).
C
C  MLAT= Number of lattices, here only cubic ones
C  MAXIR= Maximum index in one dimension of the mesh in the wedges of
C  direct space for tabulation
C  MAXSR= (2231)total number of points in the wedge including adjacent
C  points from the interior of the cube being not in the wedge
C  MAX2SR= (9261) total number of points in the octant incl. adjacent
C  MINR,MAXR= defines the cube of Ewald tabulation
C  DR= coordinate distance between two tabulated points
C  VEWTAB= Ewald lattice potential as tabulated in VEWTAB.DAT
C  ERFC0 = -2/sqrt(PI)
C  ALPHA(1:3) = cut-off parameter for the Ewald sums for sc(1) and
C  bcc(3) lattice
C  UCV(1:3) = unit cell volume for cube edge equal 1.0_dp
C  EWALD_DEN_ARR = array to hold the Ewald values in order to
C  integrate for the finite-size correction
C  TABLE = .true. if Ewald potential is obtained from tabulated values
       implicit none
       integer,parameter        :: MAXIR=31,MAXEW=10
       integer,parameter,public :: MAXSR=MAXIR*(MAXIR+1)*(MAXIR+8)/6-2
       integer,parameter,public :: MAX2SR=MAXIR*MAXIR*MAXIR
       real(dp),dimension(MAXSR,MLAT),public :: VEWTAB
       real(dp),dimension(MAX2SR,MLAT),public :: VEW2TAB
       real(dp),parameter              :: VMAD=-2.8372974794807639
       real(dp),parameter,dimension(3) :: UCV=(/1._dp,0.5_dp,0.25_dp/)
       real(dp),parameter              :: ERFC0=-1.1283791670955126_dp
       real(dp),parameter,dimension(3)       ::
     &    ALPHA=(/1.757432209_dp,2.79421047226_dp,2.22152265002_dp/)
       real(dp),dimension(3,(2*MAXEW+1)**3) :: RLAT,GLAT
       real(dp)                              :: VEW_CAL,VEW_CON
       real(dp),dimension(3)                 :: VEWGR_CAL
       real(dp),dimension(NDEN,NDEN,NDEN)    :: EWALD_DEN_ARR
       logical,public                        :: TABLE
       real(dp),parameter,public :: MINR=0.0,MAXR=0.5,
     &                       DR=((MAXR-MINR)/dble(MAXIR-1))
       public :: INITVEWALD,EWALD,DERFC,MKVEWDAT,VEW,WEDGE,MKWEDGE,
     &           OCTANT,VEWGR
C
      contains
C-----------------------------------------------------------------------
      subroutine INITVEWALD
C  la = corresponds to SULATTY=(1,2,3), here =1 because of sc lattice
C  rla =reciprocal lattice of la
       integer,parameter,dimension(3)    :: rla=(/1,3,2/)
       real(dp),parameter,dimension(3)   :: gc=(/2._dp,4._dp,4._dp/)
       integer                           :: la,mx,m1,m2,m3,n,i,ig,mi
       real(dp)                          :: dn
       real(dp),dimension(3)             :: dummy
       real(dp),dimension(4)             :: hd
       logical                           :: present
       la=SULATTY
       mx=MAXEW
       call MAKE_LATTICE(la,2*mx+1,RLAT,GLAT,mx,mi)
C       call MAKE_LATTICE(rla(la),mx,GLAT)
C       GLAT = gc(la)*PI*GLAT
       write(*,'(1a,1i2,1f7.3,1i3)')'Ewald: lattice type (1=SC),constant
     & ALPHA(la), no. of lattice vectors per axis =',la,ALPHA(la),MAXEW
C  If the table is used for the Ewald potential
       TABLE = .true.
       write (*,*) 'Ewald is taken from TABLE=',TABLE
       if (TABLE) then
        inquire (file='VEWTAB.DAT',exist=present)
        if (.not. present) then
         inquire(file='VEWTABPOS.DAT',exist=present)
         if (.not. present) call MKWEDGE
         call MKVEWDAT
        end if
        ig = 1
        open(45,file='VEWTAB.DAT',status='old')
        do i=1,MAXSR
        read (45,*) VEWTAB(i,ig)
        end do
        write (*,*) 'Initial VEW OK !'
        close(45)
       end if
C  The real space mesh for the Hartree integration of finite-size
C  correction is constructed and an array
C  for the Ewald values is stored; hd=-0.5 does not belong to WSC
       write(*,'(1a,1i4,1a)')'real space mesh for local density,
     & per axis direction NDEN =',NDEN,' points'
       n=NDEN
       dn=dble(n)
       do m1=1,n
        do m2=1,n
         do m3=1,n
           hd(1) = -0.5_dp + dble(m1)/dn
           hd(2) = -0.5_dp + dble(m2)/dn
           hd(3) = -0.5_dp + dble(m3)/dn
           hd(4) = dsqrt( sum (hd(1:3)**2))
           if (hd(4) == 0._dp) then
C  For the limit r=0 of (EWALD_DEN_ARR-1/r)=>VMAD; but to deal
C  with a finite value we suppress the 1/r part here. Has to be taken
C  into account when EWALD_DEN_ARR is used for zero argument.
            EWALD_DEN_ARR(m1,m2,m3) = VMAD
            cycle
           end if
C           call EWALD(hd(1:3),EWALD_DEN_ARR(m1,m2,m3),dummy)
           EWALD_DEN_ARR(m1,m2,m3) = VEW(SULATTY,hd(1:3))
         end do
        end do
       end do
      end subroutine INITVEWALD
C-----------------------------------------------------------------------
      subroutine EWALD(rj,vew,vewgr)
C  Ewald formula programmed as used in Kent et al. eq.2,
C  (error in Drummond et al. eq. 6), Fraser et al. eq.16
       real(dp),intent(in),dimension(3) :: rj
       real(dp),intent(out) :: vew
       real(dp),intent(out),dimension(3) :: vewgr
       integer :: la,mx,i,l,mxi
       real(dp),dimension(3,(2*MAXEW+1)**3) :: rl,gl
       real(dp) :: dir,rec,h,hh,hr1,hr2,hr3,vr,hg1,hg2,vg
       real(dp),dimension(3) :: hhgr,hgr,hgg,vgr,vgg,dirgr,recgr
C       write(*,*)'ewald_mod: IE,rj=',IE,rj
       la=SULATTY
       mx=MAXEW
       rl=RLAT
       gl=GLAT
C  Ewald sums: direct and reciprocal
       dir = 0._dp
       rec = 0._dp
       dirgr = 0
       recgr = 0
       mxi=(2*mx+1)**3
       do l=1,mxi
C  direct lattice sum:
        hr2 = dsqrt (sum ((rj(1:3)-rl(1:3,l))**2))
        hr2 = max (hr2,EMACH)
        vr = DERFC(ALPHA(la)*hr2)/hr2
        dir = dir + vr
C  gradient direct lattice
        hgr(1:3) = (rj(1:3)-rl(1:3,l))/hr2**3
        vgr(1:3) = DERFC(ALPHA(la)*hr2)+
     &     2._dp*ALPHA(la)/dsqrt(PI)*hr2*dexp(-(ALPHA(la)*hr2)**2)
        vgr = - vgr*hgr
        dirgr = dirgr + vgr
       end do
       do l=1,mxi
C  reciprocal lattice sum:
        hg1 = sum (gl(1:3,l)**2)
        if (hg1 == 0._dp) cycle
        hg2 = dcos (dot_product (gl(1:3,l),rj(1:3)))
        vg = 4._dp*PI/UCV(la)*hg2/hg1*dexp(-hg1/4._dp/ALPHA(la)**2)
        rec = rec + vg
C  gradient reciprocal lattice
        hgg = gl(1:3,l)*dsin (dot_product (gl(1:3,l),rj(1:3)))
        vgg = - 4._dp/UCV(la)*PI*hgg/hg1*
     &         dexp(-hg1/4._dp/ALPHA(la)**2)
        recgr = recgr + vgg
       end do
       h = -PI/ALPHA(la)**2/UCV(la)
       vew = dir + rec + h
       vewgr = dirgr + recgr
      end subroutine EWALD
C-----------------------------------------------------------------------
      function DERFC(x)
       real(dp) :: DERFC
       real(dp),intent(in) :: x
C  calculates error function complement according to [Abramowitz,Stegun]
       real(dp) :: T,ERFC
       real(dp),parameter :: A1=0.254829592_dp,A2=-0.284496736_dp,
     &  A3=1.421413741,A4=-1.453152027_dp,A5=1.061405429_dp,
     &  P=0.3275911_dp
        T = 1.0_dp/(P*X+1.0_dp)
        ERFC =           A5
        ERFC =  ERFC*T + A4
        ERFC =  ERFC*T + A3
        ERFC =  ERFC*T + A2
        ERFC =  ERFC*T + A1
        ERFC =  ERFC*T
        DERFC =  ERFC * EXP(-X*X)
      end function DERFC
C-----------------------------------------------------------------------
      subroutine WEDGE(rj,ir,wr)
C  Calculates the corners of a cubic mesh which enclose a specific
C  image of the point rj put to the reduced wedge of the superlattice
C  sc cell. The image is obtained from a mapping by cubic
C  transformations, see module header for a description.
       real(dp),intent(in),dimension(3) :: rj
       integer,intent(out),dimension(8) :: ir
       real(dp),intent(out),dimension(8) :: wr
C  rj= position in sc superlattice cube [-DR(MAXIR-1),+DR(MAXIR-1)]^3
C  ir= mesh index for the 8 corners of the enclosing cube in the 1/6th
C  wedge. Mesh points adjacent but beyond the surface are included, but
C  not beyond index MAXIR, to include the full cubes necessary to
C  interpolate also the surface points.
C  wr= 8 weights for interpolating the true position from the corners.
C  The weights and the index of the 8 mesh points are output.
        integer :: i,i1,i2,i3,hhh1,hhh2
        integer,dimension(3,0:1) :: iq
        integer,dimension(1) :: himax,himin
        real(dp),dimension(3) :: rq,hh
        real(dp),dimension(3,0:1) :: wq
C  Put rj into the positive octant
        rq = abs(rj)
C  Exchange coordinates for sorting into the wedge rq(1)>=rq(2)>=rq(3)
        himax = maxloc (rq)
        himin = minloc (rq)
        hh(1) = rq(himax(1))
        hh(3) = rq(himin(1))
        do i=1,3
         if ((i == himax(1)) .or. (i == himin(1))) cycle
         hh(2) = rq(i)
        end do
        rq = hh
C  Choose the mesh points iq enclosing point rq and determine the
C  distance wq between them and rq in units of DR. The indices count
C  from 1 upwards, e.g. rq=(0,0,0) being associated with iq=(1,1,1).
C  Define the i=1,8 corner points of the rq enclosing cube by the
C  coordinates.
C       (iq(1,0),iq(2,0),iq(3,0)) for i=1
C       (iq(1,0),iq(2,0),iq(3,1)) for i=2
C       (iq(1,0),iq(2,1),iq(3,0)) for i=3
C       (iq(1,0),iq(2,1),iq(3,1)) for i=4
C       (iq(1,1),iq(2,0),iq(3,0)) for i=5
C       (iq(1,1),iq(2,0),iq(3,1)) for i=6
C       (iq(1,1),iq(2,1),iq(3,0)) for i=7
C       (iq(1,1),iq(2,1),iq(3,1)) for i=8
C
        iq(1:3,0) = int(rq(1:3)/DR+EMACH) + 1
        iq(1:3,1) = iq(1:3,0) + 1
        wq(1:3,0) = (DR*dble(iq(1:3,0))-rq(1:3))/DR
        wq(1:3,1) = (rq(1:3)-DR*dble(iq(1:3,0)-1))/DR
C  Put the mesh points above MAXIR onto the border with
C  weight zero keeping only that with weight one which was on
C  the surface.
        do i=1,3
         if (iq(i,1) > MAXIR) then
          iq(i,0:1) = MAXIR
          wq(i,0) = 1.0_dp
          wq(i,1) = 0.0_dp
         end if
        end do
C  Associate its mesh index with each of the 8 corner points
C  of the enclosing cube. This can be seen also from the definition of
C  the whole mesh when the 3-dimensional indes runs over all values
C  of the reduced wedge of the total cube. The index value iq(1:3,0)
C  refers to the basic corner of the small cubes. Write this
C  index shorthand as (iq(1),iq(2),iq(3)) which runs according to:
C  i=0
C  do iq(1)=1,MAXIR
C     iq(2)=1,(iq(1)+1)    without (MAXIR+1)
C     iq(3)=1,(iq(2)+1)   without (iq(1)+2)
C     i=i+1
C     ir(i)=
C     iq(3)+((iq(2)+2)*(iq(2)-1))/2+((iq(1)-1)*(iq(1)*(iq(1)+7)+6)/6
C  end do
C  This recovers the position <--> index association.
C  The weights are chosen from linear interpolation in each coordinate
C  direction, i.e. as the product of the coordinates' deviation
C  (in units of DR)of the point under consideration from a corner point
C  Any function f we have : f(rj(1:3))=\sum^8_i=1 wr(i)*f(r_{ir(i)}) as
C  a sum over the corners.
        i = 0
        do i1=0,1
         do i2=0,1
          do i3=0,1
           i = i+1
C           hhh1 = ((iq(2,i2)+2)*(iq(2,i2)-1))/2
C           hhh2 = ((iq(1,i1)-1)*(iq(1,i1)*(iq(1,i1)+7)+6))/6
           ir(i) = iq(3,i3)
     &        + ((iq(2,i2)+2)*(iq(2,i2)-1))/2
     &        + ((iq(1,i1)-1)*(iq(1,i1)*(iq(1,i1)+7)+6))/6
           wr(i) = wq(1,i1)*wq(2,i2)*wq(3,i3)
C           if ((dabs(rj(1)) < 0.01) .and.(dabs(rj(2)-0.025) < 0.01))
C     &        then
C            write(*,'(1x,1a3,3i3,1a2,6i5)')'iq=',i1,i2,i3,'--',iq(1,i1)
C     &           ,iq(2,i2),iq(3,i3),hhh1,hhh2,ir(i)
C           end if
          end do
         end do
        end do
C        if ((dabs(rj(1)) < 0.01) .and.(dabs(rj(2)-0.025) < 0.01)) then
C         write(43,'(1x,1a3,3f7.3)') 'rj=',rj
C         write(43,'(1x,1a3,8i3)') 'ir=',ir
C         write(43,'(1x,1a3,8f7.3)') 'wr=',wr
C        end if
      end subroutine WEDGE
C-----------------------------------------------------------------------
      real(dp) function VEW(latty,rj)
C  Intermediate function to obtain the Ewald potential which is
C  calculated via the table, i.e. TABLE=.true., or directly via the
C  Ewald sum, \sum_R [1/|rj-R|], over all lattice points
C  of lattice type latty. See the routines for defining
C  the arbitrary constant and the asymptotics of finite size.
C  Works for sc,fcc,bcc lattices (LATTY=1,2,3).
C
C  Interpolation uses subroutine WEDGE which needs the
C  array VEWTAB containing the values on a simple cubic mesh.
C  The 8 corners of the embedding cube are linearly interpolated.
C  Direct calculation is called by subroutine EWALD.
C
C  Observe that VEW has limiting value, rj-->0: lim(VEW-1/|rj|)=0,
C  i.e. VEW is not finite at |rj|=0.
       real(dp) :: VEW
       integer,intent(in) :: latty
       real(dp),intent(in),dimension(3) :: rj
C
       integer,dimension(8)  :: ir
       real(dp)              :: hvew,h
       real(dp),dimension(3) :: hvewgr,hrj
       real(dp),dimension(8) :: wr
       hrj = rj
       call FOLDTOWSC(hrj)
       h = dsqrt (sum (hrj(1:3)**2))
       if (h == 0._dp) then
        write (*,*) 'error: no Ewald potential at 0'
        stop
       end if
       if (TABLE) then
C  Make the table periodic
C        call FOLDTOWSC(hrj)
        call WEDGE(hrj,ir,wr)
C        write(*,'(1x,4f7.3,1i5)')hrj,h,ir(1)
        hvew = sum (wr(1:8)*VEWTAB(ir(1:8),latty))
C        write(*,'(1x,1f9.5)')hvew
C        hvew = hvew + 1._dp/h + VMAD
        hvew = hvew + 1._dp/h
C        write(*,'(1x,1f9.5)')hvew
       else
        call EWALD(hrj,hvew,hvewgr)
       end if
       VEW = hvew
      end function VEW
C-----------------------------------------------------------------------
      subroutine MKWEDGE
C  Output R:  This makes the interpolation mesh in the wedge as
C  one-dimensional array
C  of equidistant points rq(3,ind) with index ind of a simple cubic
C  lattice with lattice constant DR and size MAXIR per dimension.
C  A sixth of the cube [0.0,DR*(MAXIR-1)]^3, where R(1,.)>=R(2,.)>=R(3,.)
C  is valid, is needed plus some border points to be able to interpolate
C  the points near the border. R(3,.) varies fastest, and R(1,.) slowest.
C  The points are stored in the file VEWTABPOS.DAT.
       integer i1,i2,i3,k2,k3,ind
       real(dp),dimension(3,MAXSR) :: rq
       write (*,'(1x,1a,i10,1a)')'total no of positions MAXSR=',MAXSR,
     & 'for the tabulation wedge'
       do i1=1,MAXIR
        k2 = i1+1
        if (k2 > MAXIR) k2=MAXIR
        do i2=1,k2
         k3 = i2+1
         if (k3 > k2) k3=k2
         do i3=1,k3
          ind = i3 + ((i2+2)*(i2-1))/2 + ((i1-1)*(i1*(i1+7)+6))/6
          rq(1,ind) = (i1-1)*DR
          rq(2,ind) = (i2-1)*DR
          rq(3,ind) = (i3-1)*DR
         end do
        end do
       end do
       open(41,file='VEWTABPOS.DAT',status='new')
       write (41,*) MAXSR
       write (41,'(1x,3f12.5)') (rq(1:3,ind),ind=1,MAXSR)
       close(41)
      end subroutine MKWEDGE
C-----------------------------------------------------------------------
      subroutine MKVEWDAT
C  Calculates and stores the Ewald data in VEWTAB.DAT
       integer                      :: i1,i2,i3,ind,npos
       real(dp)                     :: vew,h,hh
       real(dp),dimension(3)        :: dummy3
       real(dp),dimension(3,MAXSR)  :: rq
       open (unit=40,file='VEWTAB.DAT',status='new')
       open (unit=38,file='VEWTABPOS.DAT',status='old')
       read (38,*) npos
       write (*,*) 'number of positions in the VEW table =',npos
       do ind=1,MAXSR
        read (38,*) rq(1:3,ind)
        h=dsqrt (sum (rq(1:3,ind)**2))
        if (h == 0._dp) then
         hh = VMAD
        else
         call EWALD(rq(1:3,ind),vew,dummy3)
         hh=vew - 1._dp/h
        end if
        write(40,'(1x,1f15.9)') hh
       end do
       close(40)
       close(38)
       end subroutine MKVEWDAT
C-----------------------------------------------------------------------
      subroutine OCTANT(rj,ir,wr)
C  Simpler than WEDGE which would be only 1/6th of the octant
C  which is used here. But this program is for the derivative of the
C  Ewald potential which needs a coordinate exchange to obtain all
C  three gradient components from the one which is tabulated. In
C  transforming back respective minus signs apply also because of
C  treating only one octant.
C  Calculates the corners of a cubic mesh which enclose a specific
C  point rj of the superlattice sc cell. It uses from cubic symmetry
C  the mirrors of the coordinate axes to calculate only one octant
C  of the cube. The origin (0,0,0)
C  is shifted to index ir(1)=1
       real(dp),intent(in),dimension(3) :: rj
       integer,intent(out),dimension(8) :: ir
       real(dp),intent(out),dimension(8) :: wr
C
       integer :: i,i1,i2,i3
       integer,dimension(3,0:1) :: iq
       real(dp),dimension(3) :: rq
       real(dp),dimension(3,0:1) :: wq
C  Rotate the point into the positive octant of the sc superlattice
C  unit cell and determine the small cube of the interpolation mesh.
       rq(1:3) = dabs(rj(1:3))
       iq(1:3,0) = int(rq(1:3)/DR) + 1
       iq(1:3,1) = iq(1:3,0) + 1
       wq(1:3,0) = (DR*dble(iq(1:3,0))-rq(1:3))/DR
       wq(1:3,1) = (rq(1:3)-DR*dble(iq(1:3,0)-1))/DR
       do i=1,3
        if (iq(i,1) > MAXIR) then
         iq(i,0:1) = MAXIR
         wq(i,0) = 1.0_dp
         wq(i,1) = 0.0_dp
        end if
       end do
C  Here the index ir(i) is counted by i1 as the smallest unit contrary
C  to the WEDGE routine where i3 counts the smallest steps.
       i = 0
       do i1=0,1
        do i2=0,1
         do i3=0,1
          i = i+1
          ir(i) = ((iq(3,i3)-1)*MAXIR+(iq(2,i2)-1))*MAXIR + iq(1,i1)
          wr(i) = wq(1,i1)*wq(2,i2)*wq(3,i3)
         end do
        end do
       end do
      end subroutine OCTANT
C-----------------------------------------------------------------------
      subroutine VEWGR(latty,rj,grad)
C  Calculates the gradient of VEW, i.e. \sum_R [(rj-R)/|rj-R|^3],
C  over all lattice points of lattice type latty. In contrast to VEW,
C  the routine uses the full cube without reducing to the wedge as
C  it is done in VEW. Note that only one component has been tabulated.
C  The two remaining components have to be obtained by the cubic group,
C  i.e. in simple words by an axes exchange.
       integer,intent(in) :: latty
       real(dp),intent(in),dimension(3) :: rj
       real(dp),intent(out),dimension(3) :: grad
C
       integer :: ii
       integer,dimension(8) :: ir,hi0
       integer,dimension(3,8) :: hi,hir
       real(dp),dimension(8) :: wr
C
       call OCTANT(rj,ir,wr)
C  Coordinates are exchanged to obtain the whole gradient.
C  Note the integer arithmetic.
C  The 1st component has been already calculated from the table
       hir(1,1:8) = ir(1:8)
C  First the 2nd component is generated
       hi0(1:8) = ir(1:8) -1
       hi(1,1:8) = mod (hi0(1:8),MAXIR)
       hi0(1:8) = hi0(1:8)/MAXIR
       hi(2,1:8) = mod (hi0(1:8),MAXIR)
       hi0(1:8) = hi0(1:8)/MAXIR
       hi(3,1:8) = mod (hi0(1:8),MAXIR)
       hi0(1:8) = hi(1,1:8)
       hi(1,1:8) = hi(2,1:8)
       hi(2,1:8) = hi0(1:8)
       hir(2,1:8) = (hi(3,1:8)*MAXIR+hi(2,1:8))*MAXIR + hi(1,1:8) + 1
C  At last, the 3rd component is generated
       hi0(1:8) = ir(1:8) -1
       hi(1,1:8) = mod (hi0(1:8),MAXIR)
       hi0(1:8) = hi0(1:8)/MAXIR
       hi(2,1:8) = mod (hi0(1:8),MAXIR)
       hi0(1:8) = hi0(1:8)/MAXIR
       hi(3,1:8) = mod (hi0(1:8),MAXIR)
       hi0(1:8) = hi(1,1:8)
       hi(1,1:8) = hi(3,1:8)
       hi(3,1:8) = hi0(1:8)
       hir(3,1:8) = (hi(3,1:8)*MAXIR+hi(2,1:8))*MAXIR + hi(1,1:8) + 1
C  Now we are ready for the whole gradient
       do ii=1,3
        grad(ii) = sum (wr(1:8)*VEW2TAB(hir(ii,1:8),latty))
       end do
C       if (rj(1:3) < 0.0_dp) grad(1:3)=-grad(1:3)
C       grad = merge (-grad,+grad,mask = grad<0)
       grad = merge (-grad,+grad,mask = rj<0)
      end subroutine VEWGR
C-----------------------------------------------------------------------
      end module ewald_cal_tab
C-----------------------------------------------------------------------
C-----------------------------------------------------------------------
